import { createRouter, createWebHistory } from "vue-router";
import Middleware from "@/js/contexts/shared/models/Middleware";
import authRoutes from "@/js/contexts/auth/router";
import pagesRoutes from "@/js/contexts/pages/router";
import dashboardRoutes from "@/js/contexts/dashboard/router";
import linksRoutes from "@/js/contexts/links/router";
import adminRoutes from "@/js/contexts/admin/router";
import sharedRoutes from "@/js/contexts/shared/router";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import type { Permission } from "@/js/contexts/shared/models/Permission";
import useSnowflake from "@/js/contexts/shared/composables/useSnowflake";

declare module "vue-router" {
  interface RouteMeta {
    middleware?: Middleware[];
    permissions?: Permission[];
  }
}

const router = createRouter({
  history: createWebHistory(),
  routes: [...pagesRoutes, ...authRoutes, ...linksRoutes, ...dashboardRoutes, ...adminRoutes, ...sharedRoutes],
});

router.beforeResolve(async (to, from, next) => {
  const { props } = useSnowflake();
  const { isAuthenticated, authenticatedUser, fetchUser, logout, hasPermission } = useAuth();
  if (to.meta.middleware?.includes(Middleware.auth)) {
    if (isAuthenticated.value) {
      if (!authenticatedUser.value) {
        try {
          await fetchUser();
          return next();
        } catch (e) {
          logout();
          return next({ name: "login", query: { redirect: !["/login", "/", ""].includes(to.fullPath) ? to.fullPath : undefined } });
        }
      }
      if (to.meta.permissions) {
        if (!hasPermission(...to.meta.permissions)) {
          return next({ name: props.links.homepage ? "index" : "dashboard" });
        }
      }
      return next();
    }
    logout();
    return next({ name: "login", query: { redirect: !["/login", "/", ""].includes(to.fullPath) ? to.fullPath : undefined } });
  } else if (to.meta.middleware?.includes(Middleware.guest)) {
    if (isAuthenticated.value) {
      return next({ name: props.links.homepage ? "index" : "dashboard" });
    }
  }
  return next();
});

export default router;
